<?php
/*
  A set of utilities for tracking text-based game releases
  Copyright (C) 2017-2018  Alexander Yakovlev

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
namespace App\Sources;

use \App\Models\Game;
use \App\Models\Platform;
use \App\Models\Language;
use \App\Models\Author;
use \App\Models\Tag;
use \App\Source;
use Log;

class IFDB extends Source {
  public $title = "IFDB";
  public $keyword = 'ifdb';
  protected $urls = [];

  public function parse() {
    $service = new \Sabre\Xml\Service();
    $xml = $this->get_text("http://ifdb.org/allnew-rss");
    $xml = tidy_repair_string($xml, [
      'output-xml' => true,
      'input-xml' => true
    ]);
    $service->elementMap = [
      '{}item' => function(\Sabre\Xml\Reader $reader) {
        $game = new Game;
        $author = '';
        $keyValue = \Sabre\Xml\Deserializer\keyValue($reader, '{}item');
        if (isset($keyValue['{}title'])) {
          $title = $keyValue['{}title'];
          if (strpos($title, 'A new listing') === FALSE)
            return [];
          $title = str_replace('A new listing for ', '', $title);
          $title = explode(' by ', $title);
          $game->title = $title[0];
          if (isset($title[1])) {
            $author = $title[1];
          }
        }
        if (isset($keyValue['{}link'])) {
          $game->url = trim($keyValue['{}link']);
          $id = str_replace('http:', 'https:', $game->url);
          $id = str_replace('https://ifdb.org/viewgame?id=', '', $id);
          $game->source_id = $id;
        }
        $game = $this->findGame($game);
        if (isset($keyValue['{}description'])) {
          $game->description = $keyValue['{}description'];
        }
        if (isset($keyValue['{}pubDate'])) {
          $game->release_date = new \DateTime($keyValue['{}pubDate']);
        }
        if ($game->save() && !empty($author)) {
          $author_model = Author::findByName($author);
          if (empty($author_model)) {
            $author_model = new Author();
            $author_model->name = $author;
            $author_model->save();
          }
          $game->authors()->syncWithoutDetaching([$author_model->id]);
        }
        return $game;
      },
    ];
    $dom = $service->parse($xml);
  }
}
